import random
import string

def random_string(length:int, letters:bool=True, numbers:bool=False, symbols:bool=False):
    """
    Generates a random string of a given length.
    -> length*: int => The length of the string to be generated.
    -> letters: bool (True) => Whether to include letters.
    -> numbers: bool (False) => Whether to include numbers.
    -> symbols: bool (False) => Whether to include symbols.
    ----
    => The generated string.
    """

    content = ""

    if letters:
        content += string.ascii_letters

    if numbers:
        content += string.digits
        
    if symbols:
        content += string.punctuation
    
    return ''.join(random.choice(content) for _ in range(length))



import os
from sys import platform
from .argsguard import args_guard

if os.name == "posix":
    OS = "MACOSX" if platform == "darwin" else "LINUX"
    LINUX = False if platform == "darwin" else True
    MACOSX = True if platform == "darwin" else False
    WINDOWS = False
else:
    OS = "WINDOWS"
    LINUX = False
    MACOSX = False
    WINDOWS = True

def cls() -> None:
    """
    A way to clear the console screen
    """
    os.system("clear") if os.name == "posix" else os.system("cls")

clear = cls

def extract_file_type(file_location:str) -> str:
    """
    A function to return the type of file
    -> file_location: str = location of a file in string... ex : "C:\\abc\\abc\\file.xyz"
    ----
    => str: string of the file type, ex : "xyz"
    """

    if not isinstance(file_location,str):
        raise TypeError("file_location must be a string")

    try:
        return file_location.rsplit(".", 1)[1]
    except IndexError:
        raise ValueError(f"Invalid File Location : '{file_location}'")

def go_up_dir(dir_location:str) -> str:
    """
    A function to return the parent directory
    -> dir_location: str = location of a directory in string... ex : "C:\\abc\\cdef"
    ----
    => str: string of the parent directory, ex : "C:\\abc"
    """

    if not isinstance(dir_location, str):
        raise TypeError("dir_location must be a string")

    try:
        return dir_location.rsplit("/", 1)[0] if os.name == "posix" else dir_location.rsplit("\\", 1)[0]
    except Exception:
        raise ValueError(f"Invalid Directory Location : '{dir_location}'")

def dir2list(dir_location:str) -> list:
    """
    A function to return a given directory in the form of a list
    -> dir_location: str = location of a directory in string... ex : "C:\\abc\\cdef"
    ----
    => list: ex : ["C:", "abc", "cdef"]
    """

    if not isinstance(dir_location, str):
        raise TypeError("dir_location must be a string")

    try:
        return dir_location.split("/") if os.name == "posix" else dir_location.split("\\")
    except Exception:
        raise ValueError(f"Invalid Directory Location : '{dir_location}'")

def list2dir(dir_list:list) -> str:
    """
    A function to return a given list in the form of a directory
    -> dir_list: list = location of a directory in string... ex : ["C:", "abc", "bcd"]
    ----
    => str: ex : "C:\\abc\\bcd"
    """

    if not isinstance(dir_list, list):
        raise TypeError("dir_list must be a list")

    try:
        return "/".join(dir_list) if os.name == "posix" else "\\".join(dir_list)
    except Exception:
        raise ValueError(f"Invalid Directory Location : '{dir_list}'")



from hashlib import sha256
from uuid import uuid4
from .argsguard import args_guard

def hash(text: str) -> str:
    """
    Hashes a given text using the well known sha256 algorithm
    -> text: str = The string to be hashed
    ----
    => str: the hashed string
    """

    if not isinstance(text, str):
        raise TypeError("text must be a string")

    salt = uuid4().hex
    return sha256(salt.encode() + text.encode()).hexdigest() + ':' + salt

def validate_hash(hashed_text: str, check_text: str):
    """
    Checks if a given text matches a hashed(using sha256) text
    -> hashed_text: str = The hashed text
    -> check_text: str = The text which is to be checked with the hashed text
    ----
    => bool: Boolean value of : if the check_text matches hashed_text
    """

    if not isinstance(hashed_text, str):
        raise TypeError("hashed_text must be a string")

    if not isinstance(check_text, str):
        raise TypeError("check_text must be a string")

    try:
        text, salt = hashed_text.split(':')
        return text == sha256(salt.encode() + check_text.encode()).hexdigest()
    except Exception:
        return False


from functools import wraps
from inspect import isclass
from types import UnionType

def args_guard(*types):
    """A function decorator to static/type check the positional arguments passed into a function.
    -> *types: type = A bunch of types/classes which will be type checked with a function while calling it.
    ----
    => returns decorator_guard: function
    """
    def decorator_guard(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            for i in range(len(types)):
                if isclass(types[i]) or isinstance(types[i], UnionType):
                    if not isinstance(args[i], types[i]):
                        raise ValueError(f"{args[i]!r} is not a type of '{types[i]}'")
                else:
                    print(type(types[i]))
                    raise ValueError(f"{types[i]!r} is not a class/object")

            result = func(*args, **kwargs)
            return result
        return wrapper
    return decorator_guard


from .argsguard import args_guard

def parse_int(string:str) -> int:
    """
    A function to parse string characters into int
    -> string: str => a string of chars, ex : "abc12a3"
    ----
    => int: numbers present inside the string passed in, ex : 123
    """

    if not isinstance(string, str):
        raise TypeError('string must be a string')

    return int("".join([i for i in string.strip() if i.isnumeric()]))

def parse_float(string:str) -> float:
    """
    A function to parse string characters into float
    -> string: str => a string of chars, ex : "abc12a.a3"
    ----
    => float: numbers present inside the string passed in, ex : 12.3
    """

    if not isinstance(string, str):
        raise TypeError('string must be a string')

    return float("".join([i for i in string.strip() if i.isnumeric() or i == "."]))